@@ -41,8 +41,16 @@ read.dtm <- function(dtm) {
 read.slam <- function(corpus) {
   #convert a simple triplet matrix to list format.
   if(!inherits(corpus, "simple_triplet_matrix")) stop("corpus is not a simple triplet matrix")
-  documents <- ijv.to.doc(corpus$i, corpus$j, corpus$v)
-  vocab <- corpus$dimnames[[2]]
+  if ("TermDocumentMatrix" %in% class(corpus)) {
+    non_empty_docs <- which(slam::col_sums(corpus) != 0)
+    documents <- ijv.to.doc(corpus[,non_empty_docs]$j, corpus[,non_empty_docs]$i, corpus[,non_empty_docs]$v) 
+    names(documents) <- corpus[,non_empty_docs]$dimnames$Docs
+   } else {
+    non_empty_docs <- which(slam::row_sums(corpus) != 0)
+    documents <- ijv.to.doc(corpus[non_empty_docs,]$i, corpus[non_empty_docs,]$j, corpus[non_empty_docs,]$v) 
+    names(documents) <- corpus[non_empty_docs,]$dimnames$Docs
+  }
+  vocab <- corpus$dimnames$Terms
   return(list(documents=documents,vocab=vocab))
 }
 
